from sqlalchemy import (Column, String)
from sqlalchemy.orm import relationship

from .base_model import BaseModel


class HouseManagementCompany(BaseModel):
    """  Справочник упарввляющихх компаний"""

    __tablename__ = 'house_management_company'
    __table_args__ = {"extend_existing": True}

    name = Column(String, nullable=False)
    description = Column(String, nullable=False)
    address = Column(String, nullable=False)
    phone = Column(String, nullable=False)

    estate_complexes = relationship('EstateComplexes', back_populates='house_management_company', uselist=True)



from sqlalchemy import (Column, Integer, String, ForeignKey, Float, PrimaryKeyConstraint)
from sqlalchemy.orm import relationship, backref

from .base_model import BaseModel


class EstatesServices(BaseModel):
    """  Таблица связи сервисов и недвижимости """

    __tablename__ = 'estates_services'

    estate_complex_id = Column(Integer, ForeignKey('estate_complexes.id'))
    service_categories_id = Column(Integer, ForeignKey('service_categories.id'))

    estate_complexes = relationship('EstateComplexes', backref=backref('estate_complexes_assoc', cascade='all, delete-orphan'))
    service_categories = relationship('ServiceCategories', backref=backref('service_categories_assoc', cascade='all, delete-orphan'))

    __table_args__ = (
        # PrimaryKeyConstraint('estate_complex_id', 'service_categories_id'),
        {"extend_existing": True},
    )

    def __str__(self):
        return f'{self.permission} for {self.role}'


class EstateComplexes(BaseModel):
    """  Справочник объектов недвижимости """

    __tablename__ = 'estate_complexes'
    __table_args__ = {"extend_existing": True}

    estate_complex_name = Column(String, nullable=False)
    house_management_company_id = Column(Integer, ForeignKey('house_management_company.id'), nullable=False)

    house_management_company = relationship('HouseManagementCompany', back_populates='estate_complexes', uselist=False)
    service_categories = relationship('ServiceCategories', secondary='estates_services')
    buildings = relationship('Buildings', back_populates='estate_complex', uselist=True)
    apartments = relationship('Apartments', back_populates='apartment_complex',  order_by='Apartments.apartment_number', uselist=True)


class Buildings(BaseModel):
    """  Справочник сооружений комплекса """

    __tablename__ = 'buildings'
    __table_args__ = {"extend_existing": True}

    building_name = Column(String, nullable=False)
    estate_complexes_id = Column(Integer, ForeignKey('estate_complexes.id'), nullable=False)

    estate_complex = relationship('EstateComplexes', back_populates='buildings', uselist=False)
    apartments = relationship('Apartments', back_populates='building', uselist=True)


class Apartments(BaseModel):
    """  Справочник помещений """

    __tablename__ = 'apartments'
    __table_args__ = {"extend_existing": True}

    apartment_number = Column(Integer, nullable=False)
    apartment_type = Column(String, nullable=False)
    apartment_area = Column(Float, nullable=False)
    hallway_area = Column(Float, nullable=False)
    tenants_number = Column(Integer)
    estate_complexes_id = Column(Integer, ForeignKey('estate_complexes.id'), nullable=False)
    building_id = Column(Integer, ForeignKey('buildings.id'), nullable=False)

    apartment_complex = relationship('EstateComplexes', back_populates='apartments', uselist=False)
    building = relationship('Buildings', back_populates='apartments', uselist=False)



from datetime import datetime

from sqlalchemy import (Column, Integer, DateTime)
from sqlalchemy.orm import registry

mapper_registry = registry()
Base = mapper_registry.generate_base()


class BaseModel(Base):
    __abstract__ = True

    id = Column(Integer, nullable=False, unique=True, primary_key=True, autoincrement=True)
    created_at = Column(DateTime, nullable=False, default=datetime.now)
    updated_at = Column(DateTime, nullable=False, default=datetime.now, onupdate=datetime.now)



from sqlalchemy import (Column, Integer, String, Boolean, ForeignKey)
from sqlalchemy.orm import relationship

from .base_model import BaseModel


class ServiceCategories(BaseModel):
    """  Справочник сервисов"""

    __tablename__ = 'service_categories'
    __table_args__ = {"extend_existing": True}

    name = Column(String, nullable=False)
    description = Column(String)
    is_active = Column(Boolean, default=True)
    parent_category_id = Column(Integer, ForeignKey('service_categories.id'), nullable=True)

    services = relationship('Services', back_populates='service_categories', uselist=True)
    estate_complexes = relationship('EstateComplexes', secondary='estates_services')

    def __repr__(self):
        return self.name


class Services(BaseModel):
    """ Услуги """

    __tablename__ = 'services'
    __table_args__ = {"extend_existing": True}

    name = Column(String, nullable=False)
    description = Column(String)
    is_active = Column(Boolean, default=True)
    payment_required = Column(Boolean, default=False)
    need_validation = Column(Boolean, default=True)
    service_data_schema = Column(String)
    service_categories_id = Column(Integer, ForeignKey('service_categories.id'), nullable=False)

    service_categories = relationship('ServiceCategories', back_populates='services', uselist=False)
    service_item = relationship('ServiceItems')



    def __repr__(self):
        return self.name


class ServiceItems(BaseModel):
    """ Доп атрибуты услуги """

    __tablename__ = 'service_items'
    __table_args__ = {"extend_existing": True}

    name = Column(String, nullable=False)
    services_id = Column(Integer, ForeignKey('services.id'), nullable=False)



from setuptools import setup

setup(name='livingmodels',
      version='0.9.1',
      description='Models for global admin',
      packages=['livingmodels'],
      author_email='777koba@mail.ru',
      install_requires=[
            'requests', 'SQLAlchemy == 1.4.39',
            'importlib-metadata; python_version == "3.10"',
      ],
      zip_safe=False)
